#pragma once


#ifdef _WIN32
#define GET_HIGH_RESOLUTION_TIME(result) QueryPerformanceFrequency(&result)
#define SLEEP Sleep
#else
typedef int64_t LONGLONG;
typedef uint32_t DWORD;
typedef int32_t LONG;

typedef union _LARGE_INTEGER {
  struct {
    DWORD LowPart;
    LONG HighPart;
  };
  struct {
    DWORD LowPart;
    LONG HighPart;
  } u;
  LONGLONG QuadPart;
} LARGE_INTEGER;


#include <mach/mach_time.h>

#define GET_HIGH_RESOLUTION_TIME(result) ({ result.QuadPart = mach_absolute_time(); })
#define SLEEP sleep
#endif


class CDuration
{
protected:
	LARGE_INTEGER m_liStart;
	LARGE_INTEGER m_liStop;

	LONGLONG m_llFrequency;
	LONGLONG m_llCorrection;

public:
	CDuration(void);

	void Start(void);
	void Stop(void);
	double GetDuration(void) const;
	double GetDurationLive(void) const;
};

inline CDuration::CDuration(void)
{
	LARGE_INTEGER liFrequency;

	GET_HIGH_RESOLUTION_TIME(liFrequency);
	m_llFrequency = liFrequency.QuadPart;

	// Calibration
	Start();
	Stop();

	m_llCorrection = m_liStop.QuadPart-m_liStart.QuadPart;
}

inline void CDuration::Start(void)
{
	// Ensure we will not be interrupted by any other thread for a while
#ifdef _WIN32
	Sleep(0); // I'm pretty sure this isn't the way to prevent yourself from being interrupt, on any platform.  I don't know of an equivalent, aside from shifting thread priority to real-time, on Mac OS X.
#endif
	GET_HIGH_RESOLUTION_TIME(m_liStart);
}

inline void CDuration::Stop(void)
{
	GET_HIGH_RESOLUTION_TIME(m_liStop);
}

inline double CDuration::GetDuration(void) const
{
	return (double)(m_liStop.QuadPart-m_liStart.QuadPart-m_llCorrection)*1000000.0 / m_llFrequency;
}

inline double CDuration::GetDurationLive(void) const
{
	LARGE_INTEGER liTempStop;
	GET_HIGH_RESOLUTION_TIME(liTempStop);
	return (double)(liTempStop.QuadPart-m_liStart.QuadPart-m_llCorrection)*1000000.0 / m_llFrequency;
}